@@ -20,15 +20,12 @@
 #define STDC_HEADERS 1
 #define _GNU_SOURCE 1
 #define HAVE_ALLOCA_H 1
-#define HAVE_AIO_H 1
 #define HAVE_ARPA_INET_H 1
 #define HAVE_ASM_MSR_H 1
 #define HAVE_ASM_TERMBITS_H 1
-#define HAVE_BSEARCH 1
 #define HAVE_CRYPT_H 1
 #define HAVE_CURSES_H 1
 /* #undef HAVE_BFD_H */
-/* #undef HAVE_NCURSES_H */
 /* #undef HAVE_NDIR_H */
 #define HAVE_DIRENT_H 1
 #define HAVE_DLFCN_H 1
@@ -36,7 +33,6 @@
 #define HAVE_FCNTL_H 1
 #define HAVE_FENV_H 1
 #define HAVE_FLOAT_H 1
-/* #undef HAVE_FLOATINGPOINT_H */
 #define HAVE_FNMATCH_H 1
 #define HAVE_FPU_CONTROL_H 1
 #define HAVE_GRP_H 1
@@ -45,6 +41,7 @@
 #define HAVE_INTTYPES_H 1
 /* #undef HAVE_KQUEUE */
 #define HAVE_LIMITS_H 1
+#define HAVE_LINK_H 1
 #define HAVE_LINUX_UNISTD_H 1
 #define HAVE_LOCALE_H 1
 #define HAVE_MALLOC_H 1
@@ -52,7 +49,6 @@
 #define HAVE_NETINET_IN_H 1
 #define HAVE_PATHS_H 1
 #define HAVE_POLL_H 1
-/* #undef HAVE_PORT_H */
 #define HAVE_PWD_H 1
 #define HAVE_SCHED_H 1
 /* #undef HAVE_SELECT_H */
@@ -63,11 +59,9 @@
 #define HAVE_STRINGS_H 1
 #define HAVE_STRING_H 1
 #define HAVE_STDINT_H 1
-#define HAVE_SEMAPHORE_H 1
 /* #undef HAVE_SYNCH_H */
 /* #undef HAVE_SYSENT_H */
 #define HAVE_SYS_DIR_H 1
-#define HAVE_SYS_CDEFS_H 1
 #define HAVE_SYS_FILE_H 1
 /* #undef HAVE_SYS_FPU_H */
 #define HAVE_SYS_IOCTL_H 1
@@ -86,13 +80,14 @@
 #define HAVE_SYS_UTSNAME_H 1
 #define HAVE_SYS_STAT_H 1
 /* #undef HAVE_SYS_STREAM_H */
-/* #undef HAVE_SYS_TERMCAP_H */
-/* #undef HAVE_SYS_TIMEB_H */
+#define HAVE_SYS_SYSCALL_H 1
+#define HAVE_SYS_TIMEB_H 1
 #define HAVE_SYS_TIMES_H 1
 #define HAVE_SYS_TIME_H 1
 #define HAVE_SYS_TYPES_H 1
 #define HAVE_SYS_UN_H 1
 /* #undef HAVE_SYS_VADVISE_H */
+#define HAVE_SYS_STATVFS_H 1
 #define HAVE_UCONTEXT_H 1
 #define HAVE_TERM_H 1
 /* #undef HAVE_TERMBITS_H */
@@ -103,21 +98,15 @@
 #define HAVE_UNISTD_H 1
 #define HAVE_UTIME_H 1
 /* #undef HAVE_VARARGS_H */
-/* #undef HAVE_VIS_H */
 /* #undef HAVE_SYS_UTIME_H */
 #define HAVE_SYS_WAIT_H 1
 #define HAVE_SYS_PARAM_H 1
 
 /* Libraries */
-#define HAVE_LIBPTHREAD 1
-#define HAVE_LIBM 1
-#define HAVE_LIBDL 1
-#define HAVE_LIBRT 1
-/* #undef HAVE_LIBSOCKET */
-/* #undef HAVE_LIBNSL */
-#define HAVE_LIBCRYPT 1
-/* #undef HAVE_LIBMTMALLOC */
 /* #undef HAVE_LIBWRAP */
+/* #undef HAVE_SYSTEMD */
+/* #undef HAVE_CRC32_VPMSUM */
+
 /* Does "struct timespec" have a "sec" and "nsec" field? */
 /* #undef HAVE_TIMESPEC_TS_SEC */
 
@@ -132,43 +121,33 @@
 /* #undef FIONREAD_IN_SYS_FILIO */
 
 /* Functions we may want to use. */
+/* #undef HAVE_ACCEPT4 */
 #define HAVE_ACCESS 1
 /* #undef HAVE_AIOWAIT */
 #define HAVE_ALARM 1
 #define HAVE_ALLOCA 1
 /* #undef HAVE_BFILL */
-/* #undef HAVE_BMOVE */
-#define HAVE_BZERO 1
 #define HAVE_INDEX 1
-#define HAVE_CHOWN 1
 #define HAVE_CLOCK_GETTIME 1
 #define HAVE_CRYPT 1
 #define HAVE_CUSERID 1
-#define HAVE_CXX_NEW 1
-/* #undef HAVE_DIRECTIO */
-#define HAVE_DLADDR 1
 #define HAVE_DLERROR 1
 #define HAVE_DLOPEN 1
-/* #undef HAVE_DOPRNT */
 #define HAVE_FCHMOD 1
 #define HAVE_FCNTL 1
-/* #undef HAVE_FCONVERT */
 #define HAVE_FDATASYNC 1
 #define HAVE_DECL_FDATASYNC 1
 #define HAVE_FEDISABLEEXCEPT 1
 #define HAVE_FESETROUND 1
 #define HAVE_FINITE 1
 /* #undef HAVE_FP_EXCEPT */
-/* #undef HAVE_FPSETMASK */
 #define HAVE_FSEEKO 1
 #define HAVE_FSYNC 1
 #define HAVE_FTIME 1
-#define HAVE_GETADDRINFO 1
+#define HAVE_GETIFADDRS 1
 #define HAVE_GETCWD 1
 #define HAVE_GETHOSTBYADDR_R 1
 /* #undef HAVE_GETHRTIME */
-#define HAVE_GETLINE 1
-#define HAVE_GETNAMEINFO 1
 #define HAVE_GETPAGESIZE 1
 #define HAVE_GETPASS 1
 /* #undef HAVE_GETPASSPHRASE */
@@ -182,11 +161,6 @@
 /* #undef gmtime_r */
 #define HAVE_IN_ADDR_T 1
 #define HAVE_INITGROUPS 1
-/* #undef HAVE_ISSETUGID */
-#define HAVE_GETUID 1
-#define HAVE_GETEUID 1
-#define HAVE_GETGID 1
-#define HAVE_GETEGID 1
 #define HAVE_ISNAN 1
 #define HAVE_ISINF 1
 #define HAVE_LARGE_PAGE_OPTION 1
@@ -194,7 +168,6 @@
 #define HAVE_LRAND48 1
 #define HAVE_LOCALTIME_R 1
 #define HAVE_LOG2 1
-#define HAVE_LONGJMP 1
 #define HAVE_LSTAT 1
 #define HAVE_MEMALIGN 1
 /* #define HAVE_MLOCK 1 see Bug#54662 */
@@ -202,7 +175,6 @@
 #define HAVE_NL_LANGINFO 1
 #define HAVE_MADVISE 1
 #define HAVE_DECL_MADVISE 1
-/* #undef HAVE_DECL_TGOTO */
 /* #undef HAVE_DECL_MHA_MAPSIZE_VA */
 #define HAVE_MALLINFO 1
 #define HAVE_MEMCPY 1
@@ -213,36 +185,29 @@
 #define HAVE_MMAP64 1
 #define HAVE_PERROR 1
 #define HAVE_POLL 1
-/* #undef HAVE_PORT_CREATE */
 #define HAVE_POSIX_FALLOCATE 1
+#define HAVE_LINUX_FALLOC_H 1
+/* #undef HAVE_FALLOC_PUNCH_HOLE_AND_KEEP_SIZE */
 #define HAVE_PREAD 1
 #define HAVE_PAUSE_INSTRUCTION 1
 /* #undef HAVE_FAKE_PAUSE_INSTRUCTION */
+/* #undef HAVE_HMT_PRIORITY_INSTRUCTION */
 /* #undef HAVE_RDTSCLL */
 /* #undef HAVE_READ_REAL_TIME */
 /* #undef HAVE_PTHREAD_ATTR_CREATE */
 #define HAVE_PTHREAD_ATTR_GETGUARDSIZE 1
 #define HAVE_PTHREAD_ATTR_GETSTACKSIZE 1
-/* #undef HAVE_PTHREAD_ATTR_SETPRIO */
-/* #undef HAVE_PTHREAD_ATTR_SETSCHEDPARAM */
 #define HAVE_PTHREAD_ATTR_SETSCOPE 1
 #define HAVE_PTHREAD_ATTR_SETSTACKSIZE 1
 /* #undef HAVE_PTHREAD_CONDATTR_CREATE */
-#define HAVE_PTHREAD_CONDATTR_SETCLOCK 1
 #define HAVE_PTHREAD_KEY_DELETE 1
-#define HAVE_PTHREAD_KEY_DELETE 1
-/* #undef HAVE_PTHREAD_KILL */
+#define HAVE_PTHREAD_KILL 1
 #define HAVE_PTHREAD_RWLOCK_RDLOCK 1
-/* #undef HAVE_PTHREAD_SETPRIO_NP */
-/* #undef HAVE_PTHREAD_SETSCHEDPARAM */
 #define HAVE_PTHREAD_SIGMASK 1
-/* #undef HAVE_PTHREAD_THREADMASK */
 /* #undef HAVE_PTHREAD_YIELD_NP */
 #define HAVE_PTHREAD_YIELD_ZERO_ARG 1
 #define PTHREAD_ONCE_INITIALIZER PTHREAD_ONCE_INIT
 #define HAVE_PUTENV 1
-#define HAVE_RE_COMP 1
-#define HAVE_REGCOMP 1
 #define HAVE_READDIR_R 1
 #define HAVE_READLINK 1
 #define HAVE_REALPATH 1
@@ -251,15 +216,10 @@
 /* #undef HAVE_RWLOCK_INIT */
 #define HAVE_SCHED_YIELD 1
 #define HAVE_SELECT 1
-/* #undef HAVE_SETFD */
 #define HAVE_SETENV 1
 #define HAVE_SETLOCALE 1
 #define HAVE_SETUPTERM 1
-#define HAVE_SIGADDSET 1
-#define HAVE_SIGEMPTYSET 1
-#define HAVE_SIGHOLD 1
 #define HAVE_SIGSET 1
-#define HAVE_SIGSET_T 1
 #define HAVE_SIGACTION 1
 /* #undef HAVE_SIGTHREADMASK */
 #define HAVE_SIGWAIT 1
@@ -268,38 +228,24 @@
 #define HAVE_STPCPY 1
 #define HAVE_STRERROR 1
 #define HAVE_STRCOLL 1
-#define HAVE_STRSIGNAL 1
-/* #undef HAVE_STRLCPY */
-/* #undef HAVE_STRLCAT */
-/* #undef HAVE_FGETLN */
 #define HAVE_STRNLEN 1
 #define HAVE_STRPBRK 1
-/* #undef HAVE_STRSEP */
-#define HAVE_STRSTR 1
 #define HAVE_STRTOK_R 1
-#define HAVE_STRTOL 1
 #define HAVE_STRTOLL 1
 #define HAVE_STRTOUL 1
 #define HAVE_STRTOULL 1
-#define HAVE_SHMAT 1
-#define HAVE_SHMCTL 1
-#define HAVE_SHMDT 1
-#define HAVE_SHMGET 1
 /* #undef HAVE_TELL */
 #define HAVE_TEMPNAM 1
 /* #undef HAVE_THR_SETCONCURRENCY */
 /* #undef HAVE_THR_YIELD */
 #define HAVE_TIME 1
 #define HAVE_TIMES 1
-#define HAVE_VALLOC 1
 #define HAVE_VIDATTR 1
 #define HAVE_VIO_READ_BUFF 1
 #define HAVE_VASPRINTF 1
-#define HAVE_VPRINTF 1
 #define HAVE_VSNPRINTF 1
 #define HAVE_FTRUNCATE 1
 #define HAVE_TZNAME 1
-#define HAVE_AIO_READ 1
 /* Symbols we may use */
 /* #undef HAVE_SYS_ERRLIST */
 /* used by stacktrace functions */
@@ -308,25 +254,18 @@
 #define HAVE_BACKTRACE_SYMBOLS 1
 #define HAVE_BACKTRACE_SYMBOLS_FD 1
 /* #undef HAVE_PRINTSTACK */
-#define HAVE_STRUCT_SOCKADDR_IN6 1
-#define HAVE_STRUCT_IN6_ADDR 1
-/* #undef HAVE_NETINET_IN6_H */
 #define HAVE_IPV6 1
 /* #undef ss_family */
 /* #undef HAVE_SOCKADDR_IN_SIN_LEN */
 /* #undef HAVE_SOCKADDR_IN6_SIN6_LEN */
-/* #undef HAVE_TIMESPEC_TS_SEC */
-#define STRUCT_DIRENT_HAS_D_INO 1
-/* #undef STRUCT_DIRENT_HAS_D_NAMLEN */
 #define STRUCT_TIMESPEC_HAS_TV_SEC 1
 #define STRUCT_TIMESPEC_HAS_TV_NSEC 1
-#define SPRINTF_RETURNS_INT 1
 
 #define USE_MB 1
 #define USE_MB_IDENT 1
 
 /* this means that valgrind headers and macros are available */
-#define HAVE_VALGRIND 1
+#define HAVE_VALGRIND_MEMCHECK_H 1
 
 /* this means WITH_VALGRIND - we change some code paths for valgrind */
 /* #undef HAVE_valgrind */
@@ -353,21 +292,14 @@
 #define SIZEOF_SIZE_T 8
 #endif
 
-#define SIZEOF_CHAR 1
-#define HAVE_CHAR 1
 #define HAVE_LONG 1
 #define HAVE_CHARP 1
-#define SIZEOF_SHORT 2
-#define HAVE_SHORT 1
 #define SIZEOF_INT 4
 #define HAVE_INT 1
 #define SIZEOF_LONG_LONG 8
 #define HAVE_LONG_LONG 1
 #define SIZEOF_OFF_T 8
 #define HAVE_OFF_T 1
-#define SIZEOF_SIGSET_T 128
-#define HAVE_SIGSET_T 1
-#define HAVE_SIZE_T 1
 /* #undef SIZEOF_UCHAR */
 /* #undef HAVE_UCHAR */
 #define SIZEOF_UINT 4
@@ -386,25 +318,20 @@
 /* #undef HAVE_INT32 */
 /* #undef SIZEOF_UINT32 */
 /* #undef HAVE_UINT32 */
-#define SIZEOF_U_INT32_T 4
-#define HAVE_U_INT32_T 1
 /* #undef SIZEOF_INT64 */
 /* #undef HAVE_INT64 */
 /* #undef SIZEOF_UINT64 */
 /* #undef HAVE_UINT64 */
-/* #undef SIZEOF_BOOL */
-/* #undef HAVE_BOOL */
 
 #define SOCKET_SIZE_TYPE socklen_t
 
 #define HAVE_MBSTATE_T 1
 
-#define MAX_INDEXES 64
+#define MAX_INDEXES 64U
 
 #define QSORT_TYPE_IS_VOID 1
 #define RETQSORTTYPE void
 
-#define SIGNAL_RETURN_TYPE_IS_VOID 1
 #define RETSIGTYPE void
 #define VOID_SIGHANDLER 1
 #define HAVE_SIGHANDLER_T 1
@@ -434,37 +361,27 @@
 #define HAVE_WCHAR_H 1
 #define HAVE_LANGINFO_H 1
 #define HAVE_MBRLEN 1
-/* #undef HAVE_MBSCMP */
 #define HAVE_MBSRTOWCS 1
-#define HAVE_WCRTOMB 1
 #define HAVE_MBRTOWC 1
-#define HAVE_WCSCOLL 1
-#define HAVE_WCSDUP 1
 #define HAVE_WCWIDTH 1
-#define HAVE_WCTYPE 1
 #define HAVE_ISWLOWER 1
 #define HAVE_ISWUPPER 1
 #define HAVE_TOWLOWER 1
 #define HAVE_TOWUPPER 1
 #define HAVE_ISWCTYPE 1
 #define HAVE_WCHAR_T 1
-#define HAVE_WCTYPE_T 1
-#define HAVE_WINT_T 1
 
 
 #define HAVE_STRCASECMP 1
-#define HAVE_STRNCASECMP 1
-#define HAVE_STRDUP 1
-#define HAVE_LANGINFO_CODESET  1
 #define HAVE_TCGETATTR 1
-#define HAVE_FLOCKFILE 1
 
 #define HAVE_WEAK_SYMBOL 1
 #define HAVE_ABI_CXA_DEMANGLE 1
-
+#define HAVE_ATTRIBUTE_CLEANUP 1
 
 #define HAVE_POSIX_SIGNALS 1
 /* #undef HAVE_BSD_SIGNALS */
+
 /* #undef HAVE_SVR3_SIGNALS */
 /* #undef HAVE_V7_SIGNALS */
 /* #undef HAVE_ERR_remove_thread_state */
@@ -472,7 +389,6 @@
 /* #undef HAVE_SOLARIS_STYLE_GETHOST */
 
 /* #undef MY_ATOMIC_MODE_DUMMY */
-/* #undef MY_ATOMIC_MODE_RWLOCKS */
 #define HAVE_GCC_ATOMIC_BUILTINS 1
 /* #undef HAVE_GCC_C11_ATOMICS */
 /* #undef HAVE_SOLARIS_ATOMIC */
@@ -536,15 +452,9 @@
 /*
   MySQL features
 */
-#define LOCAL_INFILE_MODE_OFF  0
-#define LOCAL_INFILE_MODE_ON   1
-#define LOCAL_INFILE_MODE_AUTO 2
-#define ENABLED_LOCAL_INFILE LOCAL_INFILE_MODE_AUTO
-
+#define ENABLED_LOCAL_INFILE 1
 #define ENABLED_PROFILING 1
 /* #undef EXTRA_DEBUG */
-/* #undef BACKUP_TEST */
-/* #undef CYBOZU */
 /* #undef USE_SYMDIR */
 
 /* Character sets and collations */
@@ -553,7 +463,6 @@
 
 #define USE_MB 1
 #define USE_MB_IDENT 1
-/* #undef USE_STRCOLL */
 
 /* This should mean case insensitive file system */
 /* #undef FN_NO_CASE_SENSE */
@@ -599,7 +508,8 @@
 #define HAVE_CHARSET_utf32 1
 #define HAVE_UCA_COLLATIONS 1
 #define HAVE_COMPRESS 1
-
+/* #undef HAVE_EncryptAes128Ctr */
+/* #undef HAVE_EncryptAes128Gcm */
 
 /*
   Stuff that always need to be defined (compile breaks without it)
@@ -613,18 +523,10 @@
   Important storage engines (those that really need define 
   WITH_<ENGINE>_STORAGE_ENGINE for the whole server)
 */
-#define WITH_MYISAM_STORAGE_ENGINE 1
-#define WITH_MYISAMMRG_STORAGE_ENGINE 1
-#define WITH_HEAP_STORAGE_ENGINE 1
 #define WITH_INNOBASE_STORAGE_ENGINE 1
-#define WITH_XTRADB_STORAGE_ENGINE 1
-#define WITH_CSV_STORAGE_ENGINE 1
+/* #undef WITH_XTRADB_STORAGE_ENGINE */
 #define WITH_PARTITION_STORAGE_ENGINE 1
 #define WITH_PERFSCHEMA_STORAGE_ENGINE 1
-/* #undef WITH_NDBCLUSTER_STORAGE_ENGINE */
-#if (WITH_NDBCLUSTER_STORAGE_ENGINE) && !defined(EMBEDDED_LIBRARY)
-# define HAVE_NDB_BINLOG 1
-#endif
 #define WITH_ARIA_STORAGE_ENGINE 1
 #define USE_ARIA_FOR_TMP_TABLES 1
 
@@ -640,26 +542,32 @@
 /* #undef SO_EXT */
 
 #define MYSQL_VERSION_MAJOR 10
-#define MYSQL_VERSION_MINOR 0
-#define MYSQL_VERSION_PATCH 38
+#define MYSQL_VERSION_MINOR 2
+#define MYSQL_VERSION_PATCH 6
 #define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 10.0.38"
+#define PACKAGE_STRING "MySQL Server 10.2.6"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "10.0.38"
-#define VERSION "10.0.38"
+#define PACKAGE_VERSION "10.2.6"
+#define VERSION "10.2.6"
 #define PROTOCOL_VERSION 10
 
-#define MALLOC_LIBRARY "bundled jemalloc"
+#define MALLOC_LIBRARY "system"
 
 /* time_t related defines */
 
 #define SIZEOF_TIME_T 8
 /* #undef TIME_T_UNSIGNED */
 
+#ifndef EMBEDDED_LIBRARY
+/* #undef WSREP_INTERFACE_VERSION */
+#define WITH_WSREP 1
+#define WSREP_PROC_INFO 1
+#endif
+
 #ifdef _AIX
 /*
   AIX includes inttypes.h from sys/types.h
@@ -668,4 +576,21 @@
 #define __STDC_FORMAT_MACROS
 #endif
 
+/*
+  stat structure (from <sys/stat.h>) is conditionally defined
+  to have different layout and size depending on the defined macros.
+  The correct macro is defined in my_config.h, which means it MUST be
+  included first (or at least before <features.h> - so, practically,
+  before including any system headers).
+
+  Check the include order by looking at __GLIBC__ (defined in <features.h>)
+
+  But we cannot force all third-party clients/connectors to include
+  my_config.h first. So, their crashes are their responsibility,
+  we enable this check only for MariaDB sources (SAFE_MUTEX check).
+*/
+#if defined(__GLIBC__) && defined(SAFE_MUTEX)
+#error <my_config.h> MUST be included first!
+#endif
+
 #endif
