@@ -1,8 +1,8 @@
 #ifndef ITEM_GEOFUNC_INCLUDED
 #define ITEM_GEOFUNC_INCLUDED
 
-/* Copyright (c) 2000, 2010 Oracle and/or its affiliates.
-   Copyright (C) 2011, 2015 MariaDB
+/* Copyright (c) 2000, 2016 Oracle and/or its affiliates.
+   Copyright (C) 2011, 2016, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,57 +32,101 @@
 class Item_geometry_func: public Item_str_func
 {
 public:
-  Item_geometry_func() :Item_str_func() {}
-  Item_geometry_func(Item *a) :Item_str_func(a) {}
-  Item_geometry_func(Item *a,Item *b) :Item_str_func(a,b) {}
-  Item_geometry_func(Item *a,Item *b,Item *c) :Item_str_func(a,b,c) {}
-  Item_geometry_func(List<Item> &list) :Item_str_func(list) {}
+  Item_geometry_func(THD *thd): Item_str_func(thd) {}
+  Item_geometry_func(THD *thd, Item *a): Item_str_func(thd, a) {}
+  Item_geometry_func(THD *thd, Item *a, Item *b): Item_str_func(thd, a, b) {}
+  Item_geometry_func(THD *thd, Item *a, Item *b, Item *c):
+    Item_str_func(thd, a, b, c) {}
+  Item_geometry_func(THD *thd, List<Item> &list): Item_str_func(thd, list) {}
   void fix_length_and_dec();
   enum_field_types field_type() const  { return MYSQL_TYPE_GEOMETRY; }
-  Field *tmp_table_field(TABLE *t_arg);
-  bool is_null() { (void) val_int(); return null_value; }
+  Field *create_field_for_create_select(TABLE *table);
 };
 
 class Item_func_geometry_from_text: public Item_geometry_func
 {
 public:
-  Item_func_geometry_from_text(Item *a) :Item_geometry_func(a) {}
-  Item_func_geometry_from_text(Item *a, Item *srid) :Item_geometry_func(a, srid) {}
+  Item_func_geometry_from_text(THD *thd, Item *a): Item_geometry_func(thd, a) {}
+  Item_func_geometry_from_text(THD *thd, Item *a, Item *srid):
+    Item_geometry_func(thd, a, srid) {}
   const char *func_name() const { return "st_geometryfromtext"; }
   String *val_str(String *);
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_geometry_from_text>(thd, mem_root, this); }
 };
 
 class Item_func_geometry_from_wkb: public Item_geometry_func
 {
 public:
-  Item_func_geometry_from_wkb(Item *a): Item_geometry_func(a) {}
-  Item_func_geometry_from_wkb(Item *a, Item *srid): Item_geometry_func(a, srid) {}
+  Item_func_geometry_from_wkb(THD *thd, Item *a): Item_geometry_func(thd, a) {}
+  Item_func_geometry_from_wkb(THD *thd, Item *a, Item *srid):
+    Item_geometry_func(thd, a, srid) {}
   const char *func_name() const { return "st_geometryfromwkb"; }
   String *val_str(String *);
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_geometry_from_wkb>(thd, mem_root, this); }
 };
 
+
+class Item_func_geometry_from_json: public Item_geometry_func
+{
+  String tmp_js;
+public:
+  Item_func_geometry_from_json(THD *thd, Item *js): Item_geometry_func(thd, js) {}
+  Item_func_geometry_from_json(THD *thd, Item *js, Item *opt):
+    Item_geometry_func(thd, js, opt) {}
+  Item_func_geometry_from_json(THD *thd, Item *js, Item *opt, Item *srid):
+    Item_geometry_func(thd, js, opt, srid) {}
+  const char *func_name() const { return "st_geomfromgeojson"; }
+  String *val_str(String *);
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_geometry_from_json>(thd, mem_root, this); }
+};
+
+
 class Item_func_as_wkt: public Item_str_ascii_func
 {
 public:
-  Item_func_as_wkt(Item *a): Item_str_ascii_func(a) {}
+  Item_func_as_wkt(THD *thd, Item *a): Item_str_ascii_func(thd, a) {}
   const char *func_name() const { return "st_astext"; }
   String *val_str_ascii(String *);
   void fix_length_and_dec();
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_as_wkt>(thd, mem_root, this); }
 };
 
 class Item_func_as_wkb: public Item_geometry_func
 {
 public:
-  Item_func_as_wkb(Item *a): Item_geometry_func(a) {}
+  Item_func_as_wkb(THD *thd, Item *a): Item_geometry_func(thd, a) {}
   const char *func_name() const { return "st_aswkb"; }
   String *val_str(String *);
   enum_field_types field_type() const  { return MYSQL_TYPE_BLOB; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_as_wkb>(thd, mem_root, this); }
 };
 
+
+class Item_func_as_geojson: public Item_str_ascii_func
+{
+public:
+  Item_func_as_geojson(THD *thd, Item *js): Item_str_ascii_func(thd, js) {}
+  Item_func_as_geojson(THD *thd, Item *js, Item *max_dec_digits):
+    Item_str_ascii_func(thd, js, max_dec_digits) {}
+  Item_func_as_geojson(THD *thd, Item *js, Item *max_dec_digits, Item *opt):
+    Item_str_ascii_func(thd, js, max_dec_digits, opt) {}
+  const char *func_name() const { return "st_asgeojson"; }
+  void fix_length_and_dec();
+  String *val_str_ascii(String *);
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_as_geojson>(thd, mem_root, this); }
+};
+
+
 class Item_func_geometry_type: public Item_str_ascii_func
 {
 public:
-  Item_func_geometry_type(Item *a): Item_str_ascii_func(a) {}
+  Item_func_geometry_type(THD *thd, Item *a): Item_str_ascii_func(thd, a) {}
   String *val_str_ascii(String *);
   const char *func_name() const { return "st_geometrytype"; }
   void fix_length_and_dec() 
@@ -91,6 +135,8 @@ public:
     fix_length_and_charset(20, default_charset());
     maybe_null= 1;
   };
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_geometry_type>(thd, mem_root, this); }
 };
 
 
@@ -118,30 +164,36 @@ class Item_func_convexhull: public Item_geometry_func
   ch_node *new_ch_node() { return (ch_node *) res_heap.new_item(); }
   int add_node_to_line(ch_node **p_cur, int dir, const Gcalc_heap::Info *pi);
 public:
-  Item_func_convexhull(Item *a): Item_geometry_func(a),
+  Item_func_convexhull(THD *thd, Item *a): Item_geometry_func(thd, a),
     res_heap(8192, sizeof(ch_node))
     {}
   const char *func_name() const { return "st_convexhull"; }
   String *val_str(String *);
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_convexhull>(thd, mem_root, this); }
 };
 
 
 class Item_func_centroid: public Item_geometry_func
 {
 public:
-  Item_func_centroid(Item *a): Item_geometry_func(a) {}
+  Item_func_centroid(THD *thd, Item *a): Item_geometry_func(thd, a) {}
   const char *func_name() const { return "st_centroid"; }
   String *val_str(String *);
   Field::geometry_type get_geometry_type() const;
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_centroid>(thd, mem_root, this); }
 };
 
 class Item_func_envelope: public Item_geometry_func
 {
 public:
-  Item_func_envelope(Item *a): Item_geometry_func(a) {}
+  Item_func_envelope(THD *thd, Item *a): Item_geometry_func(thd, a) {}
   const char *func_name() const { return "st_envelope"; }
   String *val_str(String *);
   Field::geometry_type get_geometry_type() const;
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_envelope>(thd, mem_root, this); }
 };
 
 
@@ -170,28 +222,33 @@ class Item_func_boundary: public Item_geometry_func
   };
   Gcalc_result_receiver res_receiver;
 public:
-  Item_func_boundary(Item *a): Item_geometry_func(a) {}
+  Item_func_boundary(THD *thd, Item *a): Item_geometry_func(thd, a) {}
   const char *func_name() const { return "st_boundary"; }
   String *val_str(String *);
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_boundary>(thd, mem_root, this); }
 };
 
 
 class Item_func_point: public Item_geometry_func
 {
 public:
-  Item_func_point(Item *a, Item *b): Item_geometry_func(a, b) {}
-  Item_func_point(Item *a, Item *b, Item *srid): Item_geometry_func(a, b, srid) {}
+  Item_func_point(THD *thd, Item *a, Item *b): Item_geometry_func(thd, a, b) {}
+  Item_func_point(THD *thd, Item *a, Item *b, Item *srid):
+    Item_geometry_func(thd, a, b, srid) {}
   const char *func_name() const { return "point"; }
   String *val_str(String *);
   Field::geometry_type get_geometry_type() const;
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_point>(thd, mem_root, this); }
 };
 
 class Item_func_spatial_decomp: public Item_geometry_func
 {
   enum Functype decomp_func;
 public:
-  Item_func_spatial_decomp(Item *a, Item_func::Functype ft) :
-  	Item_geometry_func(a) { decomp_func = ft; }
+  Item_func_spatial_decomp(THD *thd, Item *a, Item_func::Functype ft):
+    Item_geometry_func(thd, a) { decomp_func = ft; }
   const char *func_name() const 
   { 
     switch (decomp_func)
@@ -208,14 +265,16 @@ public:
     }
   }
   String *val_str(String *);
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_spatial_decomp>(thd, mem_root, this); }
 };
 
 class Item_func_spatial_decomp_n: public Item_geometry_func
 {
   enum Functype decomp_func_n;
 public:
-  Item_func_spatial_decomp_n(Item *a, Item *b, Item_func::Functype ft):
-  	Item_geometry_func(a, b) { decomp_func_n = ft; }
+  Item_func_spatial_decomp_n(THD *thd, Item *a, Item *b, Item_func::Functype ft):
+    Item_geometry_func(thd, a, b) { decomp_func_n = ft; }
   const char *func_name() const 
   { 
     switch (decomp_func_n)
@@ -232,6 +291,8 @@ public:
     }
   }
   String *val_str(String *);
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_spatial_decomp_n>(thd, mem_root, this); }
 };
 
 class Item_func_spatial_collection: public Item_geometry_func
@@ -240,9 +301,9 @@ class Item_func_spatial_collection: public Item_geometry_func
   enum Geometry::wkbType coll_type; 
   enum Geometry::wkbType item_type;
 public:
-  Item_func_spatial_collection(
+  Item_func_spatial_collection(THD *thd,
      List<Item> &list, enum Geometry::wkbType ct, enum Geometry::wkbType it):
-  Item_geometry_func(list)
+  Item_geometry_func(thd, list)
   {
     coll_type=ct;
     item_type=it;
@@ -256,7 +317,7 @@ public:
       if (args[i]->fixed && args[i]->field_type() != MYSQL_TYPE_GEOMETRY)
       {
         String str;
-        args[i]->print(&str, QT_ORDINARY);
+        args[i]->print(&str, QT_NO_DATA_EXPANSION);
         str.append('\0');
         my_error(ER_ILLEGAL_VALUE_FOR_TYPE, MYF(0), "non geometric",
                  str.ptr());
@@ -264,7 +325,9 @@ public:
     }
   }
  
-  const char *func_name() const { return "st_multipoint"; }
+  const char *func_name() const { return "geometrycollection"; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_spatial_collection>(thd, mem_root, this); }
 };
 
 
@@ -272,15 +335,20 @@ public:
   Spatial relations
 */
 
-class Item_func_spatial_rel: public Item_bool_func2
+class Item_func_spatial_rel: public Item_bool_func2_with_rev
 {
 protected:
   enum Functype spatial_rel;
   String tmp_value1, tmp_value2;
+  SEL_ARG *get_mm_leaf(RANGE_OPT_PARAM *param, Field *field,
+                       KEY_PART *key_part,
+                       Item_func::Functype type, Item *value);
 public:
-  Item_func_spatial_rel(Item *a, Item *b, enum Functype sp_rel)
-   :Item_bool_func2(a, b), spatial_rel(sp_rel)
-  { }
+  Item_func_spatial_rel(THD *thd, Item *a, Item *b, enum Functype sp_rel):
+    Item_bool_func2_with_rev(thd, a, b), spatial_rel(sp_rel)
+  {
+    maybe_null= true;
+  }
   enum Functype functype() const { return spatial_rel; }
   enum Functype rev_functype() const { return spatial_rel; }
   bool is_null() { (void) val_int(); return null_value; }
@@ -291,17 +359,21 @@ public:
     return add_key_fields_optimize_op(join, key_fields, and_level,
                                       usable_tables, sargables, false);
   }
+  bool need_parentheses_in_default() { return false; }
+  Item *build_clone(THD *thd, MEM_ROOT *mem_root) { return 0; }
 };
 
 
 class Item_func_spatial_mbr_rel: public Item_func_spatial_rel
 {
 public:
-  Item_func_spatial_mbr_rel(Item *a, Item *b, enum Functype sp_rel)
-   :Item_func_spatial_rel(a, b, sp_rel)
+  Item_func_spatial_mbr_rel(THD *thd, Item *a, Item *b, enum Functype sp_rel):
+    Item_func_spatial_rel(thd, a, b, sp_rel)
   { }
   longlong val_int();
   const char *func_name() const;
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_spatial_mbr_rel>(thd, mem_root, this); }
 };
 
 
@@ -311,11 +383,13 @@ class Item_func_spatial_precise_rel: public Item_func_spatial_rel
   Gcalc_scan_iterator scan_it;
   Gcalc_function func;
 public:
-  Item_func_spatial_precise_rel(Item *a, Item *b, enum Functype sp_rel)
-   :Item_func_spatial_rel(a, b, sp_rel), collector()
+  Item_func_spatial_precise_rel(THD *thd, Item *a, Item *b, enum Functype sp_rel):
+    Item_func_spatial_rel(thd, a, b, sp_rel), collector()
   { }
   longlong val_int();
   const char *func_name() const;
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_spatial_precise_rel>(thd, mem_root, this); }
 };
 
 
@@ -326,11 +400,14 @@ class Item_func_spatial_relate: public Item_bool_func
   Gcalc_function func;
   String tmp_value1, tmp_value2, tmp_matrix;
 public:
-  Item_func_spatial_relate(Item *a, Item *b, Item *matrix)
-   :Item_bool_func(a, b, matrix)
+  Item_func_spatial_relate(THD *thd, Item *a, Item *b, Item *matrix):
+    Item_bool_func(thd, a, b, matrix)
   { }
   longlong val_int();
   const char *func_name() const { return "st_relate"; }
+  bool need_parentheses_in_default() { return false; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_spatial_relate>(thd, mem_root, this); }
 };
 
 
@@ -349,8 +426,9 @@ public:
   Gcalc_operation_reducer operation;
   String tmp_value1,tmp_value2;
 public:
-  Item_func_spatial_operation(Item *a,Item *b, Gcalc_function::op_type sp_op) :
-    Item_geometry_func(a, b), spatial_op(sp_op)
+  Item_func_spatial_operation(THD *thd, Item *a,Item *b,
+                              Gcalc_function::op_type sp_op):
+    Item_geometry_func(thd, a, b), spatial_op(sp_op)
   {}
   virtual ~Item_func_spatial_operation();
   String *val_str(String *);
@@ -359,6 +437,8 @@ public:
   {
     Item_func::print(str, query_type);
   }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_spatial_operation>(thd, mem_root, this); }
 };
 
 
@@ -406,20 +486,25 @@ protected:
   String tmp_value;
 
 public:
-  Item_func_buffer(Item *obj, Item *distance):
-    Item_geometry_func(obj, distance) {}
+  Item_func_buffer(THD *thd, Item *obj, Item *distance):
+    Item_geometry_func(thd, obj, distance) {}
   const char *func_name() const { return "st_buffer"; }
   String *val_str(String *);
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_buffer>(thd, mem_root, this); }
 };
 
 
 class Item_func_isempty: public Item_bool_func
 {
 public:
-  Item_func_isempty(Item *a): Item_bool_func(a) {}
+  Item_func_isempty(THD *thd, Item *a): Item_bool_func(thd, a) {}
   longlong val_int();
   const char *func_name() const { return "st_isempty"; }
   void fix_length_and_dec() { maybe_null= 1; }
+  bool need_parentheses_in_default() { return false; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_isempty>(thd, mem_root, this); }
 };
 
 class Item_func_issimple: public Item_int_func
@@ -429,46 +514,54 @@ class Item_func_issimple: public Item_int_func
   Gcalc_scan_iterator scan_it;
   String tmp;
 public:
-  Item_func_issimple(Item *a): Item_int_func(a) {}
+  Item_func_issimple(THD *thd, Item *a): Item_int_func(thd, a) {}
   longlong val_int();
   const char *func_name() const { return "st_issimple"; }
   void fix_length_and_dec() { decimals=0; max_length=2; }
   uint decimal_precision() const { return 1; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_issimple>(thd, mem_root, this); }
 };
 
 class Item_func_isclosed: public Item_int_func
 {
 public:
-  Item_func_isclosed(Item *a): Item_int_func(a) {}
+  Item_func_isclosed(THD *thd, Item *a): Item_int_func(thd, a) {}
   longlong val_int();
   const char *func_name() const { return "st_isclosed"; }
   void fix_length_and_dec() { decimals=0; max_length=2; }
   uint decimal_precision() const { return 1; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_isclosed>(thd, mem_root, this); }
 };
 
 class Item_func_isring: public Item_func_issimple
 {
 public:
-  Item_func_isring(Item *a): Item_func_issimple(a) {}
+  Item_func_isring(THD *thd, Item *a): Item_func_issimple(thd, a) {}
   longlong val_int();
   const char *func_name() const { return "st_isring"; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_isring>(thd, mem_root, this); }
 };
 
 class Item_func_dimension: public Item_int_func
 {
   String value;
 public:
-  Item_func_dimension(Item *a): Item_int_func(a) {}
+  Item_func_dimension(THD *thd, Item *a): Item_int_func(thd, a) {}
   longlong val_int();
   const char *func_name() const { return "st_dimension"; }
   void fix_length_and_dec() { max_length= 10; maybe_null= 1; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_dimension>(thd, mem_root, this); }
 };
 
 class Item_func_x: public Item_real_func
 {
   String value;
 public:
-  Item_func_x(Item *a): Item_real_func(a) {}
+  Item_func_x(THD *thd, Item *a): Item_real_func(thd, a) {}
   double val_real();
   const char *func_name() const { return "st_x"; }
   void fix_length_and_dec() 
@@ -476,6 +569,8 @@ public:
     Item_real_func::fix_length_and_dec();
     maybe_null= 1; 
   }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_x>(thd, mem_root, this); }
 };
 
 
@@ -483,7 +578,7 @@ class Item_func_y: public Item_real_func
 {
   String value;
 public:
-  Item_func_y(Item *a): Item_real_func(a) {}
+  Item_func_y(THD *thd, Item *a): Item_real_func(thd, a) {}
   double val_real();
   const char *func_name() const { return "st_y"; }
   void fix_length_and_dec() 
@@ -491,6 +586,8 @@ public:
     Item_real_func::fix_length_and_dec();
     maybe_null= 1; 
   }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_y>(thd, mem_root, this); }
 };
 
 
@@ -498,10 +595,12 @@ class Item_func_numgeometries: public Item_int_func
 {
   String value;
 public:
-  Item_func_numgeometries(Item *a): Item_int_func(a) {}
+  Item_func_numgeometries(THD *thd, Item *a): Item_int_func(thd, a) {}
   longlong val_int();
   const char *func_name() const { return "st_numgeometries"; }
   void fix_length_and_dec() { max_length= 10; maybe_null= 1; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_numgeometries>(thd, mem_root, this); }
 };
 
 
@@ -509,10 +608,12 @@ class Item_func_numinteriorring: public Item_int_func
 {
   String value;
 public:
-  Item_func_numinteriorring(Item *a): Item_int_func(a) {}
+  Item_func_numinteriorring(THD *thd, Item *a): Item_int_func(thd, a) {}
   longlong val_int();
   const char *func_name() const { return "st_numinteriorrings"; }
   void fix_length_and_dec() { max_length= 10; maybe_null= 1; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_numinteriorring>(thd, mem_root, this); }
 };
 
 
@@ -520,10 +621,12 @@ class Item_func_numpoints: public Item_int_func
 {
   String value;
 public:
-  Item_func_numpoints(Item *a): Item_int_func(a) {}
+  Item_func_numpoints(THD *thd, Item *a): Item_int_func(thd, a) {}
   longlong val_int();
   const char *func_name() const { return "st_numpoints"; }
   void fix_length_and_dec() { max_length= 10; maybe_null= 1; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_numpoints>(thd, mem_root, this); }
 };
 
 
@@ -531,7 +634,7 @@ class Item_func_area: public Item_real_func
 {
   String value;
 public:
-  Item_func_area(Item *a): Item_real_func(a) {}
+  Item_func_area(THD *thd, Item *a): Item_real_func(thd, a) {}
   double val_real();
   const char *func_name() const { return "st_area"; }
   void fix_length_and_dec() 
@@ -539,6 +642,8 @@ public:
     Item_real_func::fix_length_and_dec();
     maybe_null= 1; 
   }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_area>(thd, mem_root, this); }
 };
 
 
@@ -546,7 +651,7 @@ class Item_func_glength: public Item_real_func
 {
   String value;
 public:
-  Item_func_glength(Item *a): Item_real_func(a) {}
+  Item_func_glength(THD *thd, Item *a): Item_real_func(thd, a) {}
   double val_real();
   const char *func_name() const { return "st_length"; }
   void fix_length_and_dec() 
@@ -554,6 +659,8 @@ public:
     Item_real_func::fix_length_and_dec();
     maybe_null= 1; 
   }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_glength>(thd, mem_root, this); }
 };
 
 
@@ -561,10 +668,12 @@ class Item_func_srid: public Item_int_func
 {
   String value;
 public:
-  Item_func_srid(Item *a): Item_int_func(a) {}
+  Item_func_srid(THD *thd, Item *a): Item_int_func(thd, a) {}
   longlong val_int();
   const char *func_name() const { return "srid"; }
   void fix_length_and_dec() { max_length= 10; maybe_null= 1; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_srid>(thd, mem_root, this); }
 };
 
 
@@ -576,9 +685,11 @@ class Item_func_distance: public Item_real_func
   Gcalc_function func;
   Gcalc_scan_iterator scan_it;
 public:
-  Item_func_distance(Item *a, Item *b): Item_real_func(a, b) {}
+  Item_func_distance(THD *thd, Item *a, Item *b): Item_real_func(thd, a, b) {}
   double val_real();
   const char *func_name() const { return "st_distance"; }
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_distance>(thd, mem_root, this); }
 };
 
 
@@ -589,10 +700,12 @@ class Item_func_pointonsurface: public Item_geometry_func
   Gcalc_function func;
   Gcalc_scan_iterator scan_it;
 public:
-  Item_func_pointonsurface(Item *a): Item_geometry_func(a) {}
+  Item_func_pointonsurface(THD *thd, Item *a): Item_geometry_func(thd, a) {}
   const char *func_name() const { return "st_pointonsurface"; }
   String *val_str(String *);
   Field::geometry_type get_geometry_type() const;
+  Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+  { return get_item_copy<Item_func_pointonsurface>(thd, mem_root, this); }
 };
 
 
@@ -600,9 +713,16 @@ public:
 class Item_func_gis_debug: public Item_int_func
 {
   public:
-    Item_func_gis_debug(Item *a) :Item_int_func(a) { null_value= false; }
+    Item_func_gis_debug(THD *thd, Item *a): Item_int_func(thd, a)
+    { null_value= false; }
     const char *func_name() const  { return "st_gis_debug"; }
     longlong val_int();
+    bool check_vcol_func_processor(void *arg)
+    {
+      return mark_unsupported_function(func_name(), "()", arg, VCOL_IMPOSSIBLE);
+    }
+    Item *get_copy(THD *thd, MEM_ROOT *mem_root)
+    { return get_item_copy<Item_func_gis_debug>(thd, mem_root, this); }
 };
 #endif
 
